function map = lbmap(n,scheme)
%LBMAP Returns specified Light-Bertlein colormap.
%
%   LBMAP(N,SCHEME) returns an Nx3 colormap. SCHEME can be one of the
%   following strings:
%
%       'Blue'       Single-hue progression to purlish-blue (default)
%       'BlueGray'   Diverging progression from blue to gray
%       'BrownBlue'  Orange-white-purple diverging scheme
%       'RedBlue'    Modified spectral scheme
%
%   If N is not specified, the size of the colormap is determined by the
%   current figure. If no figure exists, MATLAB creates one.
%
%Example 1: 7-color single-hue blue (default)
%   load penny
%   imagesc(P)
%   colormap(lbmap(7))
%   colorbar
%
%Example 2: 11-color modified spectrum
%   load penny
%   imagesc(P)
%   colormap(lbmap(11,'RedBlue'))
%   colorbar
%
%   See also HSV, GRAY, HOT, BONE, COPPER, PINK, FLAG, COLORMAP, RGBPLOT.

% Reference:
% A. Light & P.J. Bartlein, "The End of the Rainbow? Color Schemes for
% Improved Data Graphics," Eos,Vol. 85, No. 40, 5 October 2004.
% http://geography.uoregon.edu/datagraphics/EOS/Light&Bartlein_EOS2004.pdf

% Copyright 2007-2010 The MathWorks, Inc.

%defensive programming
error(nargchk(0,2,nargin))
error(nargoutchk(0,1,nargout))

%defaults
if nargin<2
  scheme = 'Blue';
end
if nargin<1
  n = size(get(gcf,'colormap'),1);
end

%valid schemes
switch lower(scheme)
  case 'blue'
    baseMap = BlueMap;
  case 'bluegray'
    baseMap = BlueGrayMap;
  case 'brownblue'
    baseMap = BrownBlueMap;
  case 'redblue'
    baseMap = RedBlueMap;
  case 'blueyellow'
    baseMap = BlueYellowMap;
  case 'redblue2d'
    baseMap = RedBlueMap2D;
  otherwise
    error(['Invalid scheme ' scheme])
end
idx1 = linspace(0,1,size(baseMap,1));
idx2 = linspace(0,1,n);
map = interp1(idx1,baseMap,idx2);

function baseMap = BlueMap
baseMap = [243 246 248;
           224 232 240;
           171 209 236;
           115 180 224;
            35 157 213;
             0 142 205;
             0 122 192]/255;

function baseMap = BlueGrayMap
%DivergingBlueGray
baseMap = [  0 170 227;
            53 196 238;
           133 212 234;
           190 230 242;
           217 224 230;
           146 161 170;
           109 122 129;
            65  79  81]/255;

function baseMap = BrownBlueMap
baseMap = [144 100  44;
           187 120  54;
           225 146  65;
           248 184 139;
           244 218 200;
           241 244 245;
           207 226 240;
           160 190 225;
           109 153 206;
            70  99 174;
            24  79 162]/255;

function baseMap = RedBlueMap
baseMap = [175  53  71;
           216  82  88;
           239 133 122;
           245 177 139;
           249 216 168;
           242 238 197;
           216 236 241;
           154 217 238;
            68 199 239;
             0 170 226;
             0 116 188]/255;

function baseMap = BlueYellowMap
baseMap = [200	200	255;
            5	5	255;
            0	0	0;
            255	255	5;
            255	255	200]/255;
        
        
function baseMap = RedBlueMap2D
baseMap = [0	0	255;
        255	0	0;
        1	1	255;
        255	1	1;
        2	2	255;
        255	2	2;
        3	3	255;
        255	3	3;
        4	4	255;
        255	4	4;
        5	5	255;
        255	5	5;
        6	6	255;
        255	6	6;
        7	7	255;
        255	7	7;
        8	8	255;
        255	8	8;
        9	9	255;
        255	9	9;
        10	10	255;
        255	10	10;
        11	11	255;
        255	11	11;
        12	12	255;
        255	12	12;
        13	13	255;
        255	13	13;
        14	14	255;
        255	14	14;
        15	15	255;
        255	15	15;
        16	16	255;
        255	16	16;
        17	17	255;
        255	17	17;
        18	18	255;
        255	18	18;
        19	19	255;
        255	19	19;
        20	20	255;
        255	20	20;
        21	21	255;
        255	21	21;
        22	22	255;
        255	22	22;
        23	23	255;
        255	23	23;
        24	24	255;
        255	24	24;
        25	25	255;
        255	25	25;
        26	26	255;
        255	26	26;
        27	27	255;
        255	27	27;
        28	28	255;
        255	28	28;
        29	29	255;
        255	29	29;
        30	30	255;
        255	30	30;
        31	31	255;
        255	31	31;
        32	32	255;
        255	32	32;
        33	33	255;
        255	33	33;
        34	34	255;
        255	34	34;
        35	35	255;
        255	35	35;
        36	36	255;
        255	36	36;
        37	37	255;
        255	37	37;
        38	38	255;
        255	38	38;
        39	39	255;
        255	39	39;
        40	40	255;
        255	40	40;
        41	41	255;
        255	41	41;
        42	42	255;
        255	42	42;
        43	43	255;
        255	43	43;
        44	44	255;
        255	44	44;
        45	45	255;
        255	45	45;
        46	46	255;
        255	46	46;
        47	47	255;
        255	47	47;
        48	48	255;
        255	48	48;
        49	49	255;
        255	49	49;
        50	50	255;
        255	50	50;
        51	51	255;
        255	51	51;
        52	52	255;
        255	52	52;
        53	53	255;
        255	53	53;
        54	54	255;
        255	54	54;
        55	55	255;
        255	55	55;
        56	56	255;
        255	56	56;
        57	57	255;
        255	57	57;
        58	58	255;
        255	58	58;
        59	59	255;
        255	59	59;
        60	60	255;
        255	60	60;
        61	61	255;
        255	61	61;
        62	62	255;
        255	62	62;
        63	63	255;
        255	63	63;
        64	64	255;
        255	64	64;
        65	65	255;
        255	65	65;
        66	66	255;
        255	66	66;
        67	67	255;
        255	67	67;
        68	68	255;
        255	68	68;
        69	69	255;
        255	69	69;
        70	70	255;
        255	70	70;
        71	71	255;
        255	71	71;
        72	72	255;
        255	72	72;
        73	73	255;
        255	73	73;
        74	74	255;
        255	74	74;
        75	75	255;
        255	75	75;
        76	76	255;
        255	76	76;
        77	77	255;
        255	77	77;
        78	78	255;
        255	78	78;
        79	79	255;
        255	79	79;
        80	80	255;
        255	80	80;
        81	81	255;
        255	81	81;
        82	82	255;
        255	82	82;
        83	83	255;
        255	83	83;
        84	84	255;
        255	84	84;
        85	85	255;
        255	85	85;
        86	86	255;
        255	86	86;
        87	87	255;
        255	87	87;
        88	88	255;
        255	88	88;
        89	89	255;
        255	89	89;
        90	90	255;
        255	90	90;
        91	91	255;
        255	91	91;
        92	92	255;
        255	92	92;
        93	93	255;
        255	93	93;
        94	94	255;
        255	94	94;
        95	95	255;
        255	95	95;
        96	96	255;
        255	96	96;
        97	97	255;
        255	97	97;
        98	98	255;
        255	98	98;
        99	99	255;
        255	99	99;
        100	100	255;
        255	100	100;
        101	101	255;
        255	101	101;
        102	102	255;
        255	102	102;
        103	103	255;
        255	103	103;
        104	104	255;
        255	104	104;
        105	105	255;
        255	105	105;
        106	106	255;
        255	106	106;
        107	107	255;
        255	107	107;
        108	108	255;
        255	108	108;
        109	109	255;
        255	109	109;
        110	110	255;
        255	110	110;
        111	111	255;
        255	111	111;
        112	112	255;
        255	112	112;
        113	113	255;
        255	113	113;
        114	114	255;
        255	114	114;
        115	115	255;
        255	115	115;
        116	116	255;
        255	116	116;
        117	117	255;
        255	117	117;
        118	118	255;
        255	118	118;
        119	119	255;
        255	119	119;
        120	120	255;
        255	120	120;
        121	121	255;
        255	121	121;
        122	122	255;
        255	122	122;
        123	123	255;
        255	123	123;
        124	124	255;
        255	124	124;
        125	125	255;
        255	125	125;
        126	126	255;
        255	126	126;
        127	127	255;
        255	127	127;
        128	128	255;
        255	128	128;
        129	129	255;
        255	129	129;
        130	130	255;
        255	130	130;
        131	131	255;
        255	131	131;
        132	132	255;
        255	132	132;
        133	133	255;
        255	133	133;
        134	134	255;
        255	134	134;
        135	135	255;
        255	135	135;
        136	136	255;
        255	136	136;
        137	137	255;
        255	137	137;
        138	138	255;
        255	138	138;
        139	139	255;
        255	139	139;
        140	140	255;
        255	140	140;
        141	141	255;
        255	141	141;
        142	142	255;
        255	142	142;
        143	143	255;
        255	143	143;
        144	144	255;
        255	144	144;
        145	145	255;
        255	145	145;
        146	146	255;
        255	146	146;
        147	147	255;
        255	147	147;
        148	148	255;
        255	148	148;
        149	149	255;
        255	149	149;
        150	150	255;
        255	150	150;
        151	151	255;
        255	151	151;
        152	152	255;
        255	152	152;
        153	153	255;
        255	153	153;
        154	154	255;
        255	154	154;
        155	155	255;
        255	155	155;
        156	156	255;
        255	156	156;
        157	157	255;
        255	157	157;
        158	158	255;
        255	158	158;
        159	159	255;
        255	159	159;
        160	160	255;
        255	160	160;
        161	161	255;
        255	161	161;
        162	162	255;
        255	162	162;
        163	163	255;
        255	163	163;
        164	164	255;
        255	164	164;
        165	165	255;
        255	165	165;
        166	166	255;
        255	166	166;
        167	167	255;
        255	167	167;
        168	168	255;
        255	168	168;
        169	169	255;
        255	169	169;
        170	170	255;
        255	170	170;
        171	171	255;
        255	171	171;
        172	172	255;
        255	172	172;
        173	173	255;
        255	173	173;
        174	174	255;
        255	174	174;
        175	175	255;
        255	175	175;
        176	176	255;
        255	176	176;
        177	177	255;
        255	177	177;
        178	178	255;
        255	178	178;
        179	179	255;
        255	179	179;
        180	180	255;
        255	180	180;
        181	181	255;
        255	181	181;
        182	182	255;
        255	182	182;
        183	183	255;
        255	183	183;
        184	184	255;
        255	184	184;
        185	185	255;
        255	185	185;
        186	186	255;
        255	186	186;
        187	187	255;
        255	187	187;
        188	188	255;
        255	188	188;
        189	189	255;
        255	189	189;
        190	190	255;
        255	190	190;
        191	191	255;
        255	191	191;
        192	192	255;
        255	192	192;
        193	193	255;
        255	193	193;
        194	194	255;
        255	194	194;
        195	195	255;
        255	195	195;
        196	196	255;
        255	196	196;
        197	197	255;
        255	197	197;
        198	198	255;
        255	198	198;
        199	199	255;
        255	199	199;
        200	200	255;
        255	200	200;
        201	201	255;
        255	201	201;
        202	202	255;
        255	202	202;
        203	203	255;
        255	203	203;
        204	204	255;
        255	204	204;
        205	205	255;
        255	205	205;
        206	206	255;
        255	206	206;
        207	207	255;
        255	207	207;
        208	208	255;
        255	208	208;
        209	209	255;
        255	209	209;
        210	210	255;
        255	210	210;
        211	211	255;
        255	211	211;
        212	212	255;
        255	212	212;
        213	213	255;
        255	213	213;
        214	214	255;
        255	214	214;
        215	215	255;
        255	215	215;
        216	216	255;
        255	216	216;
        217	217	255;
        255	217	217;
        218	218	255;
        255	218	218;
        219	219	255;
        255	219	219;
        220	220	255;
        255	220	220;
        221	221	255;
        255	221	221;
        222	222	255;
        255	222	222;
        223	223	255;
        255	223	223;
        224	224	255;
        255	224	224;
        225	225	255;
        255	225	225;
        226	226	255;
        255	226	226;
        227	227	255;
        255	227	227;
        228	228	255;
        255	228	228;
        229	229	255;
        255	229	229;
        230	230	255;
        255	230	230;
        231	231	255;
        255	231	231;
        232	232	255;
        255	232	232;
        233	233	255;
        255	233	233;
        234	234	255;
        255	234	234;
        235	235	255;
        255	235	235;
        236	236	255;
        255	236	236;
        237	237	255;
        255	237	237;
        238	238	255;
        255	238	238;
        239	239	255;
        255	239	239;
        240	240	255;
        255	240	240;
        241	241	255;
        255	241	241;
        242	242	255;
        255	242	242;
        243	243	255;
        255	243	243;
        244	244	255;
        255	244	244;
        245	245	255;
        255	245	245;
        246	246	255;
        255	246	246;
        247	247	255;
        255	247	247;
        248	248	255;
        255	248	248;
        249	249	255;
        255	249	249;
        250	250	255;
        255	250	250;
        251	251	255;
        255	251	251;
        252	252	255;
        255	252	252;
        253	253	255;
        255	253	253;
        254	254	255;
        255	254	254;
        255	255	255;
        255	255	255;]/255;